/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2016 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.api.governance;

import com.google.common.base.Function;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.Multimaps;
import org.apache.commons.collections.CollectionUtils;

import java.util.List;

/**
 * @author mamo
 */
public class GovernanceUpdateResult {

    public enum UpdateStatus {
        create, deleteOrUpdate, error, skip;

        @Override
        public String toString() {
            return BlackDuckUtils.camelize(name());
        }
    }

    public final GovernanceRequestInfo requestInfo;
    public final UpdateStatus updateStatus;
    private String message;

    public GovernanceUpdateResult(GovernanceRequestInfo requestInfo, String updateStatus) {
        this.requestInfo = requestInfo;
        this.updateStatus = getUpdateStatus(updateStatus);
    }

    /**
     * get updated enum status
     *
     * @param updateStatus - updated status
     * @return - enum status
     */
    public UpdateStatus getUpdateStatus(String updateStatus) {
        switch (updateStatus) {
            case "create":
                return UpdateStatus.create;
            case "deleteOrUpdate":
                return UpdateStatus.deleteOrUpdate;
            case "error":
                return UpdateStatus.error;
            case "skip":
                return UpdateStatus.skip;
            default:
                return UpdateStatus.create;
        }
    }

    public GovernanceUpdateResult message(String message) {
        this.message = message;
        return this;
    }

    public String getMessage() {
        return message;
    }

    public static int numberOfErrors(List<GovernanceUpdateResult> resultList) {
        if (CollectionUtils.isNotEmpty(resultList)) {
            ImmutableList<GovernanceUpdateResult> errors = Multimaps.index(resultList, $updateStatus).get(
                    UpdateStatus.error);
            return errors != null ? errors.size() : 0;
        } else {
            return 0;
        }
    }

    public static final Function<GovernanceUpdateResult, UpdateStatus> $updateStatus = new Function<GovernanceUpdateResult, UpdateStatus>() {
        @Override
        public UpdateStatus apply(GovernanceUpdateResult input) {
            return input.updateStatus;
        }
    };

}
