/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2016 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.api.maven;

import org.artifactory.api.repo.Async;
import org.artifactory.repo.RepoPath;

/**
 * A service for calculating maven metadata.
 *
 * @author Yossi Shaul
 */
public interface MavenMetadataService {

    /**
     * Calculate maven metadata on the giver repo path asynchronously.
     *
     * @param baseFolderPath A path to a folder to start calculating metadata from. Must be a local non-cache repository
     *                       path.
     * @param recursive      True is should calculate recursively from the base folder path
     */
    @Async(delayUntilAfterCommit = true)
    void calculateMavenMetadataAsync(RepoPath baseFolderPath, boolean recursive);

    /**
     * Calculates the maven metadata recursively on all the folders under the input folder.
     * This will also trigger asynchronous maven metadata calculation for maven plugins.
     *
     * @param baseFolderPath Base repo path to start calculating from
     * @param recursive      True is should calculate recursively from the base folder path
     */
    void calculateMavenMetadata(RepoPath baseFolderPath, boolean recursive);

    /**
     * Calculate the maven plugins metadata asynchronously after the current transaction is committed. The reason is the
     * metadata calculator uses xpath queries for its job and since the move is not committed yet, the xpath query
     * result might not be accurate (for example when moving plugins from one repo to another the query on the source
     * repository will return the moved plugins while the target repo will not return them). <p/> Note: you should call
     * the markBaseForMavenMetadataRecalculation() before calling this method to recover in case this task is
     * interrupted in the middle.
     *
     * @param localRepoKey Key of the local non-cache repository to calculate maven plugins metadata on.
     */
    @Async(delayUntilAfterCommit = true)
    void calculateMavenPluginsMetadataAsync(String localRepoKey);

}
