/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2016 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.security;

import org.artifactory.common.Info;

import java.util.Set;

/**
 * Date: 8/2/11
 * Time: 10:50 AM
 *
 * @author Fred Simon
 */
public interface UserInfo extends Info {
    String MISSION_CONTROL = "Mission-Control";
    String ANONYMOUS = "anonymous";

    String getUsername();

    String getPassword();

    String getSalt();

    String getEmail();

    String getPrivateKey();

    String getPublicKey();

    String getGenPasswordKey();

    boolean isAdmin();

    boolean isEnabled();

    boolean isUpdatableProfile();

    boolean isAccountNonExpired();

    boolean isAccountNonLocked();

    boolean isTransientUser();

    String getRealm();

    boolean isExternal();

    /**
     * Indicates whether the user's credentials (password) has expired. Expired credentials prevent
     * authentication.
     *
     * @return <code>false</code> if the user's credentials are valid (ie non-expired), <code>true</code> if no longer
     *         valid (ie expired)
     */
    boolean isCredentialsExpired();

    boolean isCredentialsNonExpired();

    boolean isAnonymous();

    boolean isInGroup(String groupName);

    Set<UserGroupInfo> getGroups();

    Set<UserPropertyInfo> getUserProperties();

    long getLastLoginTimeMillis();

    String getLastLoginClientIp();
    @Deprecated
    long getLastAccessTimeMillis();
    @Deprecated
    String getLastAccessClientIp();

    boolean hasSameAuthorizationContext(UserInfo o);

    boolean hasInvalidPassword();

    String getBintrayAuth();

    boolean isLocked();
}
