/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2016 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.storage.fs.lock.map;


import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author Shay Yaakov
 */
public class JVMLockingMap implements LockingMap {

    private final Map<String, String> locks = new HashMap<>();

    @Override
    public boolean tryAddAndLock(String path, long timeout, TimeUnit timeUnit) throws InterruptedException {
        long startTime = System.currentTimeMillis();
        while (true) {
            synchronized (locks) {
                if (locks.containsKey(path)) {
                    long userTimeout = timeUnit.toMillis(timeout);
                    locks.wait(userTimeout);
                    long endTime = System.currentTimeMillis();
                    if (endTime - startTime >= userTimeout) {
                        return false;
                    }
                } else {
                    locks.put(path, path);
                    return true;
                }
            }
        }
    }

    public static void main(String[] args) {
        long l = TimeUnit.MILLISECONDS.toMillis(5000);
        System.out.println(l);
    }
    @Override
    public void removeAndUnlock(String path) {
        synchronized (locks) {
            locks.remove(path);
            locks.notifyAll();
        }
    }

    @Override
    public void removeAndForceUnlock(String path) {
        removeAndUnlock(path);
    }

    @Override
    public boolean isLocked(String path) {
        synchronized (locks) {
            return locks.containsKey(path);
        }
    }
}
