/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2016 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.storage.fs.lock.map;

import java.util.concurrent.TimeUnit;

/**
 * A data structure which allows only one concurrent writer and many readers.
 *
 * @author Shay Yaakov
 */
public interface LockingMap {

    /**
     * Adds a path to the map while locking other threads from writing the same path concurrently.
     *
     * @param path     The path to add
     * @param timeout  the maximum time to wait for the lock
     * @param timeUnit the time unit of the {@code time} argument
     * @return {@code true} if the lock was acquired and {@code false}
     * if the waiting time elapsed before the lock was acquired
     * @throws InterruptedException if the current thread is interrupted
     *                              while acquiring the lock (and interruption of lock
     *                              acquisition is supported)
     */
    boolean tryAddAndLock(String path, long timeout, TimeUnit timeUnit) throws InterruptedException;

    /**
     * Removes the path from the map and releases it's lock.
     *
     * @param path The path to remove
     */
    void removeAndUnlock(String path);


    /**
     * Removes the path from the map and releases it's lock regardless the thread owner.
     *
     * @param path The path to remove
     */
    void removeAndForceUnlock(String path);

    /**
     * checks if the path is locked.
     *
     * @param path The path to remove
     */
    boolean isLocked(String path);
}
