import EVENTS from '../../../../constants/artifacts_events.constants';
import FIELD_OPTIONS from "../../../../constants/field_options.constats";

export class AdminConfigurationXrayController {

    constructor($scope, XrayDao, FooterDao, JFrogModal, JFrogGridFactory, JFrogEventBus, commonGridColumns) {
        this.$scope = $scope;
        this.commonGridColumns = commonGridColumns;
        this.xrayDao = XrayDao;
        this.JFrogEventBus = JFrogEventBus;
        this.footerDao = FooterDao;
        this.artifactoryGridFactory = JFrogGridFactory;
        this.modal = JFrogModal;

        this.createGrid();
        this.getData();

//        XrayDao.getConf().$promise.then((data)=>console.log(data))
    }
    getData() {
        this.xrayDao.getIndex().$promise.then((data)=>{
            _.forEach(data, (row) => {

                let rowPackageType =_.find(FIELD_OPTIONS.repoPackageTypes, (type) => {
                    return type.value == row.pkgType.toLowerCase();
                });

                row.packageIcon = rowPackageType.icon;
                row.repoType = row.type.charAt(0).toUpperCase() + row.type.slice(1);
            });
            this.indexedRepos = data;
            this.gridOptions.setGridData(data);
        });
        this.xrayDao.getNoneIndex().$promise.then((data)=>{
            data.forEach((obj)=>{
                obj._iconClass = "icon icon-" + obj.type.toLowerCase() + "-repo";
            })
            this.unindexedRepos = data;
        });


    }

    createGrid() {
        this.gridOptions = this.artifactoryGridFactory.getGridInstance(this.$scope)
                .setColumns(this.getColumns())
                .setMultiSelect()
                .setButtons(this.getActions())
                .setRowTemplate('default')
                .setBatchActions(this.getBatchActions());
    }

    getColumns() {
        return [
            {
                name: 'Name',
                displayName: 'Name',
                field: "name",
                width: '40%'
            },
            {
                name: 'Repository Type',
                displayName: 'Repository Type',
                field: "type",
                cellTemplate: '<div class="ui-grid-cell-contents">{{row.entity.repoType}}</div>',
                width: '30%'
            },
            {
                name: 'Package Type',
                displayName: 'Package Type',
                field: "pkgType",
                cellTemplate: this.commonGridColumns.iconColumn('row.entity.pkgType', 'row.entity.packageIcon', 'repo-type-icon'),
                width: '30%'
            }
        ]
    }

    getActions() {
        return [
            {
                icon: 'icon icon-clear',
                tooltip: 'Remove',
                callback: (row) => this.removeIndexes(row)
            }

        ];
    }
    getBatchActions() {
        return [
            {
                icon: 'clear',
                name: 'Remove',
                callback: () => this.bulkRemove()
            }
        ]
    }

    add() {

        let modalScope = this.$scope.$new();
        let modalInstance;

        modalScope.availableRepos = _.cloneDeep(this.unindexedRepos);
        modalScope.reposToAdd = [];

        modalScope.save = () => {
            modalInstance.close();
            let addList = _.map(modalScope.reposToAdd,(obj) => _.pick(obj,['name','type']));
            this.xrayDao.addIndex({},addList).$promise.then(() => {
                this.getData();
            });

        };

        modalInstance = this.modal.launchModal('add_xray_index_modal',modalScope);
    }

    removeIndexes(repos) {
        if (_.isArray(repos)) {
            this.xrayDao.removeIndex({},repos).$promise.then(() => {
                this.getData();
            });
        }
        else {
            this.xrayDao.removeIndex({},[{name: repos.name,type: repos.type}]).$promise.then(() => {
                this.getData();
            });
        }
    }

    bulkRemove() {
        let selectedRows = this.gridOptions.api.selection.getSelectedRows();
        let removeList = _.map(selectedRows,(obj) => _.pick(obj,['name','type']));
        this.removeIndexes(removeList);
    }
}