import AdminState       from './admin/admin.module';
import ArtifactState    from './artifacts/artifacts.module';
import BuildsState      from './builds/builds.module';
import HomeModule       from './home/home.module';
import OAuthErrorModule from './oauth_error/oauth_error.module';
import notFound404      from './not_found_404/not_found_404.module';
import Forbidden403     from './forbidden_403/forbidden_403.module';
import Login            from './login/login.module';
import ChangePassword   from './change_password/change_password.module';
import ForgotPassword   from './forgot_password/forgot_password.module';
import UserProfile      from './user_profile/user_profile.module';
import ServerError      from './server_error/server_error.module';
import BaseState        from './base/base.module';

angular.module('artifactory.states', [
    AdminState.name,
    ArtifactState.name,
    BuildsState.name,
    HomeModule.name,
    OAuthErrorModule.name,
    notFound404.name,
    Forbidden403.name,
    Login.name,
    ChangePassword.name,
    ForgotPassword.name,
    UserProfile.name,
    ServerError.name,
    BaseState.name,
    'artifactory.services',
    'artifactory.dao',
    'cfp.hotkeys',
    'ui.router'
    ])
    .run(changeStateHook);

function changeStateHook(User, $rootScope, $q, JFrogNotifications, $location, $timeout, $state,
                         ArtifactoryFeatures, FooterDao, ArtifactoryState, JFrogEventBus) {

    let EVENTS = JFrogEventBus.getEventsDefinition();

    JFrogEventBus.register(EVENTS.USER_LOGOUT, (confirmDiscard) => {
        if (confirmDiscard) {
            checkDiscardConfirmation($q).then(()=>{
                JFrogEventBus.dispatch(EVENTS.USER_LOGOUT);
            })
        }
    });

    $rootScope.$on('$locationChangeStart', (e,newUrl)=>{
        if (ArtifactoryState.getState('confirmDiscardModalOpen')) {
            e.preventDefault();
        }
    });

    $rootScope.$on('$stateChangeStart', (e, toState, toParams, fromState, fromParams) => {

        if (fromState.name.startsWith('admin.') || fromState.name === 'user_profile') {
            if (!ArtifactoryState.getState('confirmDiscardModalOpen')) {
                checkDiscardConfirmation($q, e).then(()=> {
                    $state.go(toState.name, toParams);
                });
            }
            else {
                e.preventDefault();
                return;
            }
        }
        let saveAdminState = ArtifactoryState.getState('saveAdminState');
    if (toState.name.startsWith('admin.') && !toState.name.match(/(?:.new|.edit)\b/) && saveAdminState && !e.defaultPrevented) {
            ArtifactoryState.setState('lastAdminState', toState);
            ArtifactoryState.setState('lastAdminStateParams', toParams);
            ArtifactoryState.removeState('saveAdminState');
        }
        else if (saveAdminState && !e.defaultPrevented) {
            ArtifactoryState.removeState('saveAdminState');
        }


        if (fromState.name && toState.name && fromState.name != toState.name) {
            JFrogEventBus.dispatch(EVENTS.CANCEL_SPINNER);
        }

        if (toState.name === 'artifacts.browsers.search') {
            //MOVED FROM artifacts.module.js to prevent error message (ui-router bug workaround)
            JFrogEventBus.dispatch(EVENTS.SEARCH_URL_CHANGED, toParams);
        }
        else if (fromState.name === 'artifacts.browsers.search') {
            JFrogEventBus.dispatch(EVENTS.CLEAR_SEARCH);
        }

        if (toState.name === 'change-password' && !toParams.username) {
            e.preventDefault();
            $state.go('login');
        }

        if (toState.name === 'oauth_error') {
            e.preventDefault();

            let message = $location.search().message;
            let gotoState = localStorage.stateBeforeOAuth;

            if (gotoState === 'login') {
                $state.go(gotoState,{oauthError: message, location: "replace"});
            }
            else if (gotoState === 'user_profile') {
                JFrogNotifications.create({error: message});
                $state.go(gotoState,{location: "replace"});
            }
            else {
                JFrogNotifications.create({error: message});
                $state.go('home',{location: "replace"});
            }
        }

        if (toState.name.match(/^builds/) && !User.getCurrent().getCanDeploy()) {
            toParams.tab = 'published';
        }

        if (toState.name === 'login' && $location.path() !== '/login' && $location.path() !== '/forgot-password' && $location.path() !== '/change-password' && $location.path() !== '/oauth_error' && !$location.path().startsWith('/resetpassword') && !$location.path().startsWith('/404') && !$location.path().startsWith('/403')) {
            let afterLogin = ArtifactoryState.getState('urlAfterLogin');
            if (!afterLogin) ArtifactoryState.setState('urlAfterLogin', $location.path());
        }


        // Permissions:

        if (!User.getCurrent().canView(toState.name, toParams)) {
            if (User.getCurrent().isProWithoutLicense()) {
                $timeout(() => $location.path('admin/configuration/register_pro'));
            }else {
                if ($location.path() !== '/login') ArtifactoryState.setState('urlAfterLogin', $location.path());
                e.preventDefault();
                if (User.getCurrent().name === 'anonymous') {
                    JFrogNotifications.create({error: 'You are not authorized to view this page'});
                    $timeout(() => $location.path('/login'));
                }
                else {
                    $timeout(() => $location.path('/403'));
                }
            }
        }
        // Features per license:
        else {
            let feature = toParams.feature;
            // Must verify footer data is available before checking (for initial page load)
            FooterDao.get().then(() => {
                if (ArtifactoryFeatures.isDisabled(feature) || ArtifactoryFeatures.isHidden(feature)) {
                JFrogNotifications.create({error: 'Page unavailable'});
                    e.preventDefault();
                    $timeout(() => $location.path('/home'));
                }
            });
        }

        if (!e.defaultPrevented) {
            JFrogEventBus.dispatch(EVENTS.CLOSE_MODAL);
        }
    })
}


function checkDiscardConfirmation($q, e) {

    let defer = $q.defer();
    let forms = $('form');
    let changeDiscovered = false;
    for (let i = 0; i< forms.length; i++) {
        let form = forms[i];
        let controller = angular.element(form).controller();
        if (controller && controller._$modelSaver$_ && controller._$modelSaver$_.confirmOnLeave && !controller._$modelSaver$_.isModelSaved()) {
            changeDiscovered = true;

            controller._$modelSaver$_.ask().then(()=>{
                controller._$modelSaver$_.confirmOnLeave =   false;
                defer.resolve();
            });

            break;
        }
    }

    if (!changeDiscovered && !e) {
        defer.resolve();
    }
    else if (changeDiscovered && e) {
        e.preventDefault();
    }

    return defer.promise;

}
