package org.artifactory.rest.common.model.xray;

import org.artifactory.descriptor.repo.XrayDescriptor;
import org.artifactory.rest.common.model.BaseModel;
import org.springframework.util.StringUtils;

/**
 * @author Chen Keinan
 */
public class XrayConfigModel extends BaseModel{

    public static final String xrayApiUri = "/api/v1";

    private String xrayBaseUrl;
    private String xrayUser;
    private String xrayPass;
    private String artUser;
    private String artPass;
    private String artifactoryId;

    public XrayConfigModel(){}

    public XrayConfigModel(XrayDescriptor xrayDescriptor) {
        if (xrayDescriptor != null) {
            this.setXrayUser(xrayDescriptor.getUser());
            this.setXrayPass(xrayDescriptor.getPassword());
            this.setXrayBaseUrl(xrayDescriptor.getBaseUrl());
            this.setArtifactoryId(xrayDescriptor.getArtifactoryId());
        }
    }

    public String getXrayBaseUrl() {
        return xrayBaseUrl;
    }

    public void setXrayBaseUrl(String xrayBaseUrl) {
        this.xrayBaseUrl = xrayBaseUrl;
    }

    public String getArtUser() {
        return artUser;
    }

    public void setArtUser(String artUser) {
        this.artUser = artUser;
    }

    public String getXrayUser() {
        return xrayUser;
    }

    public void setXrayUser(String xrayUser) {
        this.xrayUser = xrayUser;
    }

    public String getXrayPass() {
        return xrayPass;
    }

    public void setXrayPass(String xrayPass) {
        this.xrayPass = xrayPass;
    }

    public String getArtifactoryId() {
        return artifactoryId;
    }

    public void setArtifactoryId(String artifactoryId) {
        this.artifactoryId = artifactoryId;
    }

    public String getArtPass() {
        return artPass;
    }

    public void setArtPass(String artPass) {
        this.artPass = artPass;
    }

    public XrayDescriptor toDescriptor(){
        XrayDescriptor xrayDescriptor = new XrayDescriptor();
        xrayDescriptor.setEnabled(true);
        xrayDescriptor.setUser(this.xrayUser);
        xrayDescriptor.setPassword(this.xrayPass);

        String xrayBaseUrl = this.xrayBaseUrl;

        if (!xrayBaseUrl.endsWith("/")){
            xrayBaseUrl = xrayBaseUrl+"/";
        }
        xrayDescriptor.setBaseUrl(xrayBaseUrl);
        xrayDescriptor.setArtifactoryId(this.getArtifactoryId());
        return xrayDescriptor;
    }

    public boolean validate() {
        if (StringUtils.isEmpty(this.xrayUser) ||
                StringUtils.isEmpty(this.xrayPass) ||
                StringUtils.isEmpty(this.xrayBaseUrl) ||
                StringUtils.isEmpty(this.artifactoryId)) {
            return false;
        }
        return true;
    }
}
