/*
 *
 * Copyright 2016 JFrog Ltd. All rights reserved.
 * JFROG PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package org.artifactory.rest.common.model.xray;

import com.google.common.collect.Maps;
import org.artifactory.addon.xray.XrayRepo;
import org.artifactory.rest.common.model.BaseModel;

import javax.annotation.Nonnull;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>Created on 29/06/16
 *
 * @author Yinon Avraham
 */
public class XrayIndexStatsModel extends BaseModel {

    private final Map<String, XrayRepoIndexStatsModel> repoStats = Maps.newHashMap();

    public XrayIndexStatsModel(@Nonnull Collection<XrayRepo> repos) {
        repos.stream().forEach(repo ->
                repoStats.put(repo.getName(), new XrayRepoIndexStatsModel(repo))
        );
    }

    public List<XrayRepoIndexStatsModel> getRepoStats() {
        //Return a predictable order (sorted by repo name)
        return repoStats.values().stream()
                .sorted((e1, e2) -> e1.repo.getName().compareTo(e2.repo.getName()))
                .collect(Collectors.toList());
    }

    public void increaseStarted(@Nonnull String repo) {
        repoStats.get(repo).increaseStarted();
    }

    public void increaseCompleted(@Nonnull String repo) {
        repoStats.get(repo).increaseCompleted();
    }

    public static class XrayRepoIndexStatsModel {
        private final XrayRepo repo;
        private long started = 0;
        private long completed = 0;

        public XrayRepoIndexStatsModel(XrayRepo repo) {
            this.repo = repo;
        }

        public XrayRepo getRepo() {
            return repo;
        }

        public void increaseStarted() {
            started++;
        }

        public long getStarted() {
            return started;
        }

        public void increaseCompleted() {
            completed++;
        }

        public long getCompleted() {
            return completed;
        }
    }
}
