package org.artifactory.rest.common.service.admin.xray;

import org.apache.http.HttpStatus;
import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.xray.XrayAddon;
import org.artifactory.api.context.ContextHelper;
import org.artifactory.rest.common.model.xray.XrayConfigModel;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.RestResponse;
import org.artifactory.rest.common.service.RestService;
import org.artifactory.security.AuthenticationHelper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

/**
 * @author Chen Keinan
 */
@Component
@Scope(BeanDefinition.SCOPE_PROTOTYPE)
public class UpdateXrayConfigService implements RestService {
    private static final Logger log = LoggerFactory.getLogger(UpdateXrayConfigService.class);

    @Override
    public void execute(ArtifactoryRestRequest request, RestResponse response) {
        AddonsManager addonsManager = ContextHelper.get().beanForType(AddonsManager.class);
        XrayAddon xrayAddon = addonsManager.addonByType(XrayAddon.class);
        if (!addonsManager.isXrayLicensed()) {
          response.error("Invalid xray license for Xray").responseCode(HttpStatus.SC_BAD_REQUEST);
        }  else {
            // create xray Config
            if (!updateXrayConfig(request, xrayAddon)) {
                response.error("Could not create Xray config - check Artifactory logs").responseCode(400);
            }
        }
    }

    /**
     * create xray config
     */
    private boolean updateXrayConfig(ArtifactoryRestRequest request, XrayAddon xrayAddon) {
        String remoteAddress = AuthenticationHelper.getRemoteAddress(AuthenticationHelper.getAuthentication());
        log.info("Updating Xray config, request received from instance at: {}", remoteAddress);
        XrayConfigModel xrayModel = (XrayConfigModel) request.getImodel();
        if (xrayModel.validate()) {
            return xrayAddon.createXrayConfig(xrayModel.toDescriptor());
        } else {
            log.debug("Invalid Xray config model received!");
            return false;
        }
    }
}
