/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2016 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.ui.rest.model.general;

import org.artifactory.descriptor.message.SystemMessageDescriptor;
import org.artifactory.rest.common.model.BaseModel;
import org.codehaus.jackson.annotate.JsonProperty;

/**
 * @author Chen Keinan
 */
public class Footer extends BaseModel {
    private String versionInfo;
    private String buildNumber;
    private String licenseInfo;
    private String copyRights;
    private String copyRightsUrl;
    private boolean isAol;
    private boolean isDedicatedAol;
    private String versionID;
    private boolean globalRepoEnabled;
    private boolean userLogo;
    private String logoUrl;
    private String serverName;
    private boolean helpLinksEnabled;
    private boolean systemMessageEnabled;
    private String systemMessageTitle;
    private String systemMessageTitleColor;
    private String systemMessage;
    private boolean showSystemMessageOnAllPages;
    private String serverId;
    private boolean trashDisabled;
    private boolean allowPermDeletes;
    private boolean samlRedirectEnabled;
    private boolean xrayConfigured;
    private boolean xrayEnabled;
    private boolean xrayLicense;

    public Footer(String licenseInfo, String versionInfo, String copyRights, String copyRightsUrl,
                  String buildNumber, boolean isAol, boolean isDedicatedAol, boolean isGlobalRepoEnabled, String versionID, boolean userLogo,
                  String logoUrl, String serverName, SystemMessageDescriptor systemMessageDescriptor,
                  boolean helpLinksEnabled, String serverId, boolean trashDisabled, boolean allowPermDeletes, boolean samlRedirectEnabled,
                  boolean xrayConfigured, boolean xrayEnabled, boolean xrayLicense) {
        this.licenseInfo = licenseInfo;
        this.versionInfo = versionInfo;
        this.copyRights = copyRights;
        this.copyRightsUrl = copyRightsUrl;
        this.buildNumber = buildNumber;
        this.isAol = isAol;
        this.isDedicatedAol = isDedicatedAol;
        this.globalRepoEnabled = isGlobalRepoEnabled;
        this.versionID = versionID;
        this.userLogo = userLogo;
        this.logoUrl = logoUrl;
        this.serverName = serverName;
        this.trashDisabled = trashDisabled;
        this.allowPermDeletes = allowPermDeletes;
        this.systemMessageEnabled = systemMessageDescriptor.isEnabled();
        this.systemMessageTitle = systemMessageDescriptor.getTitle();
        this.systemMessageTitleColor = systemMessageDescriptor.getTitleColor();
        this.systemMessage = systemMessageDescriptor.getMessage();
        this.showSystemMessageOnAllPages = systemMessageDescriptor.isShowOnAllPages();
        this.helpLinksEnabled = helpLinksEnabled;
        this.serverId = serverId;
        this.samlRedirectEnabled = samlRedirectEnabled;
        this.xrayConfigured = xrayConfigured;
        this.xrayEnabled = xrayEnabled;
        this.xrayLicense = xrayLicense;
    }

    public String getLicenseInfo() {
        return licenseInfo;
    }

    public void setLicenseInfo(String licenseInfo) {
        this.licenseInfo = licenseInfo;
    }

    public String getVersionInfo() {
        return versionInfo;
    }

    public void setVersionInfo(String versionInfo) {
        this.versionInfo = versionInfo;
    }

    public String getCopyRights() {
        return copyRights;
    }

    public void setCopyRights(String copyRights) {
        this.copyRights = copyRights;
    }

    public String getCopyRightsUrl() {
        return copyRightsUrl;
    }

    public void setCopyRightsUrl(String copyRightsUrl) {
        this.copyRightsUrl = copyRightsUrl;
    }

    public String getBuildNumber() {
        return buildNumber;
    }

    public void setBuildNumber(String buildNumber) {
        this.buildNumber = buildNumber;
    }

    @JsonProperty("isAol")
    public boolean isAol() {
        return isAol;
    }

    public String getVersionID() {
        return versionID;
    }

    public void setVersionID(String versionID) {
        this.versionID = versionID;
    }

    public boolean isGlobalRepoEnabled() {
        return globalRepoEnabled;
    }

    public boolean isUserLogo() {
        return userLogo;
    }

    public void setUserLogo(boolean userLogo) {
        this.userLogo = userLogo;
    }

    public String getLogoUrl() {
        return logoUrl;
    }

    public void setLogoUrl(String logoUrl) {
        this.logoUrl = logoUrl;
    }

    public String getServerName() {
        return serverName;
    }

    public void setServerName(String serverName) {
        this.serverName = serverName;
    }

    public boolean isSystemMessageEnabled() {
        return systemMessageEnabled;
    }

    public String getSystemMessageTitle() {
        return systemMessageTitle;
    }

    public String getSystemMessageTitleColor() {
        return systemMessageTitleColor;
    }

    public String getSystemMessage() {
        return systemMessage;
    }

    public boolean isShowSystemMessageOnAllPages() {
        return showSystemMessageOnAllPages;
    }

    public boolean isHelpLinksEnabled() {
        return helpLinksEnabled;
    }

    public String getServerId() {
        return serverId;
    }

    public void setServerId(String serverId) {
        this.serverId = serverId;
    }

    public boolean isTrashDisabled() {
        return trashDisabled;
    }

    public boolean isAllowPermDeletes() {
        return allowPermDeletes;
    }

    @JsonProperty("isDedicatedAol")
    public boolean isDedicatedAol() { return isDedicatedAol; }

    public void setDedicatedAol(boolean dedicatedAol) { isDedicatedAol = dedicatedAol; }

    public boolean isSamlRedirectEnabled() {
        return samlRedirectEnabled;
    }

    public void setSamlRedirectEnabled(boolean samlRedirectEnabled) {
        this.samlRedirectEnabled = samlRedirectEnabled;
    }

    public boolean isXrayConfigured() { return xrayConfigured; }

    public void setXrayConfigured(boolean xrayConfigured) { this.xrayConfigured = xrayConfigured; }

    public boolean isXrayEnabled() {return xrayEnabled; }

    public void setXrayEnabled(boolean xrayEnabled) {this.xrayEnabled = xrayEnabled; }

    public boolean isXrayLicense() {return xrayLicense; }

    public void setXrayLicense(boolean xrayLicense) {this.xrayLicense = xrayLicense; }
}
