/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2016 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.ui.rest.service.admin.security.group;

import org.apache.http.HttpStatus;
import org.artifactory.api.security.UserGroupService;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.RestResponse;
import org.artifactory.rest.common.service.RestService;
import org.artifactory.security.GroupInfo;
import org.artifactory.security.UserInfo;
import org.artifactory.ui.rest.common.SecurityModelPopulator;
import org.artifactory.ui.rest.model.admin.security.group.Group;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @author Chen Keinan
 * @author Dan Feldman
 */
@Component
@Scope(BeanDefinition.SCOPE_PROTOTYPE)
public class GetGroupService implements RestService {

    @Autowired
    private UserGroupService userGroupService;

    @Override
    public void execute(ArtifactoryRestRequest request, RestResponse response) {
        String groupName = request.getPathParamByKey("id");
        Group group = getGroup(groupName);
        if (group == null) {
            response.error("No such group '" + groupName + "'.").responseCode(HttpStatus.SC_NOT_FOUND);
        } else {
            addGroupUsers(groupName, group);
            response.iModel(group);
        }
    }

    private void addGroupUsers(String groupName, Group group) {
        List<UserInfo> usersInGroup = userGroupService.findUsersInGroup(groupName);
        if (usersInGroup != null) {
            group.setUsersInGroup(usersInGroup.parallelStream()
                    .map(UserInfo::getUsername)
                    .collect(Collectors.toList()));
        }
    }

    private Group getGroup(String groupName) {
        GroupInfo group = userGroupService.findGroup(groupName);
        return group == null ? null : SecurityModelPopulator.getGroupConfiguration(group);
    }
}
