package org.artifactory.rest.resource.xray;

import org.artifactory.api.security.AuthorizationService;
import org.artifactory.rest.common.model.xray.XrayConfigModel;
import org.artifactory.rest.common.resource.BaseResource;
import org.artifactory.rest.services.ConfigServiceFactory;
import org.artifactory.rest.services.RepoServiceFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import javax.annotation.security.RolesAllowed;
import javax.ws.rs.*;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

/**
 * @author Shay Yaakov
 * @author Chen Keinan
 */
@RolesAllowed({AuthorizationService.ROLE_ADMIN})
@Component
@Path("xray")
@Scope(BeanDefinition.SCOPE_PROTOTYPE)
public class XrayResource extends BaseResource {

    @Autowired
    private RepoServiceFactory repoServiceFactory;

    @Autowired
    private ConfigServiceFactory configServiceFactory;

    @POST
    @Path("index")
    @Consumes(MediaType.APPLICATION_JSON)
    public Response index() throws Exception {
        return runService(repoServiceFactory.indexXray());
    }

    @GET
    @Path("license")
    @Consumes(MediaType.APPLICATION_JSON)
    @Produces(MediaType.TEXT_PLAIN)
    public Response license() throws Exception {
        return runService(repoServiceFactory.getXrayLicense());
    }

    @GET
    @Path("indexStats")
    @Produces(MediaType.APPLICATION_JSON)
    public Response indexStats() throws Exception {
        return runService(repoServiceFactory.getXrayIndexStats());
    }

    @GET
    @Path("repos")
    @RolesAllowed({AuthorizationService.ROLE_USER, AuthorizationService.ROLE_ADMIN})
    @Produces(MediaType.APPLICATION_JSON)
    public Response getConfigureReposIndexing() throws Exception {
        return runService(configServiceFactory.getXrayConfiguredRepos());
    }

    //Configuration endpoints

    @POST
    @Produces(MediaType.APPLICATION_JSON)
    @Consumes(MediaType.APPLICATION_JSON)
    public Response createXrayConfig(XrayConfigModel xrayConfigModel) throws Exception {
        return runService(configServiceFactory.createXrayConfig(), xrayConfigModel);
    }

    @PUT
    @Produces(MediaType.APPLICATION_JSON)
    @Consumes(MediaType.APPLICATION_JSON)
    public Response updateXrayConfig(XrayConfigModel xrayConfigModel) throws Exception {
        return runService(configServiceFactory.updateXrayConfig(), xrayConfigModel);
    }

    @DELETE
    @Produces(MediaType.APPLICATION_JSON)
    public Response deleteXrayConfig() throws Exception {
        return runService(configServiceFactory.deleteXrayConfig());
    }
}
